@@ -129,12 +129,14 @@ func main() {
 	s.SetStyle(defStyle)
 	s.EnableMouse()
 	s.EnablePaste()
+	s.EnableFocus()
 	s.Clear()
 
 	posfmt := "Mouse: %d, %d  "
 	btnfmt := "Buttons: %s"
 	keyfmt := "Keys: %s"
 	pastefmt := "Paste: [%d] %s"
+	focusfmt := "Focus: %s"
 	white := tcell.StyleDefault.
 		Foreground(tcell.ColorMidnightBlue).Background(tcell.ColorLightCoral)
 
@@ -148,9 +150,10 @@ func main() {
 	pstr := ""
 	ecnt := 0
 	pasting := false
+	focus := true // assume we are focused when we start
 
 	for {
-		drawBox(s, 1, 1, 42, 7, white, ' ')
+		drawBox(s, 1, 1, 42, 8, white, ' ')
 		emitStr(s, 2, 2, white, "Press ESC twice to exit, C to clear.")
 		emitStr(s, 2, 3, white, fmt.Sprintf(posfmt, mx, my))
 		emitStr(s, 2, 4, white, fmt.Sprintf(btnfmt, bstr))
@@ -162,6 +165,12 @@ func main() {
 		}
 		emitStr(s, 2, 6, white, fmt.Sprintf(pastefmt, len(pstr), ps))
 
+		fstr := "false"
+		if focus {
+			fstr = "true"
+		}
+		emitStr(s, 2, 7, white, fmt.Sprintf(focusfmt, fstr))
+
 		s.Show()
 		bstr = ""
 		ev := s.PollEvent()
@@ -281,7 +290,7 @@ func main() {
 			switch ev.Buttons() {
 			case tcell.ButtonNone:
 				if ox >= 0 {
-					bg := theme[(lchar-'0') % 8]
+					bg := theme[lchar%8]
 					fg := tcell.ColorBlack
 					drawBox(s, ox, oy, x, y,
 						up.Background(bg).Foreground(fg),
@@ -315,6 +324,8 @@ func main() {
 			lchar = ch
 			s.SetContent(w-1, h-1, 'M', nil, st)
 			mx, my = x, y
+		case *tcell.EventFocus:
+			focus = ev.Focused
 		default:
 			s.SetContent(w-1, h-1, 'X', nil, st)
 		}
