@@ -1,4 +1,4 @@
-// Copyright 2021 The TCell Authors
+// Copyright 2024 The TCell Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use file except in compliance with the License.
@@ -167,59 +167,73 @@ type Terminfo struct {
 	// Terminal support for these are going to vary amongst XTerm
 	// emulations, so don't depend too much on them in your application.
 
-	StrikeThrough   string // smxx
-	SetFgBg         string // setfgbg
-	SetFgBgRGB      string // setfgbgrgb
-	SetFgRGB        string // setfrgb
-	SetBgRGB        string // setbrgb
-	KeyShfUp        string // shift-up
-	KeyShfDown      string // shift-down
-	KeyShfPgUp      string // shift-kpp
-	KeyShfPgDn      string // shift-knp
-	KeyCtrlUp       string // ctrl-up
-	KeyCtrlDown     string // ctrl-left
-	KeyCtrlRight    string // ctrl-right
-	KeyCtrlLeft     string // ctrl-left
-	KeyMetaUp       string // meta-up
-	KeyMetaDown     string // meta-left
-	KeyMetaRight    string // meta-right
-	KeyMetaLeft     string // meta-left
-	KeyAltUp        string // alt-up
-	KeyAltDown      string // alt-left
-	KeyAltRight     string // alt-right
-	KeyAltLeft      string // alt-left
-	KeyCtrlHome     string
-	KeyCtrlEnd      string
-	KeyMetaHome     string
-	KeyMetaEnd      string
-	KeyAltHome      string
-	KeyAltEnd       string
-	KeyAltShfUp     string
-	KeyAltShfDown   string
-	KeyAltShfLeft   string
-	KeyAltShfRight  string
-	KeyMetaShfUp    string
-	KeyMetaShfDown  string
-	KeyMetaShfLeft  string
-	KeyMetaShfRight string
-	KeyCtrlShfUp    string
-	KeyCtrlShfDown  string
-	KeyCtrlShfLeft  string
-	KeyCtrlShfRight string
-	KeyCtrlShfHome  string
-	KeyCtrlShfEnd   string
-	KeyAltShfHome   string
-	KeyAltShfEnd    string
-	KeyMetaShfHome  string
-	KeyMetaShfEnd   string
-	EnablePaste     string // bracketed paste mode
-	DisablePaste    string
-	PasteStart      string
-	PasteEnd        string
-	Modifiers       int
-	InsertChar      string // string to insert a character (ich1)
-	AutoMargin      bool   // true if writing to last cell in line advances
-	TrueColor       bool   // true if the terminal supports direct color
+	StrikeThrough           string // smxx
+	SetFgBg                 string // setfgbg
+	SetFgBgRGB              string // setfgbgrgb
+	SetFgRGB                string // setfrgb
+	SetBgRGB                string // setbrgb
+	KeyShfUp                string // shift-up
+	KeyShfDown              string // shift-down
+	KeyShfPgUp              string // shift-kpp
+	KeyShfPgDn              string // shift-knp
+	KeyCtrlUp               string // ctrl-up
+	KeyCtrlDown             string // ctrl-left
+	KeyCtrlRight            string // ctrl-right
+	KeyCtrlLeft             string // ctrl-left
+	KeyMetaUp               string // meta-up
+	KeyMetaDown             string // meta-left
+	KeyMetaRight            string // meta-right
+	KeyMetaLeft             string // meta-left
+	KeyAltUp                string // alt-up
+	KeyAltDown              string // alt-left
+	KeyAltRight             string // alt-right
+	KeyAltLeft              string // alt-left
+	KeyCtrlHome             string
+	KeyCtrlEnd              string
+	KeyMetaHome             string
+	KeyMetaEnd              string
+	KeyAltHome              string
+	KeyAltEnd               string
+	KeyAltShfUp             string
+	KeyAltShfDown           string
+	KeyAltShfLeft           string
+	KeyAltShfRight          string
+	KeyMetaShfUp            string
+	KeyMetaShfDown          string
+	KeyMetaShfLeft          string
+	KeyMetaShfRight         string
+	KeyCtrlShfUp            string
+	KeyCtrlShfDown          string
+	KeyCtrlShfLeft          string
+	KeyCtrlShfRight         string
+	KeyCtrlShfHome          string
+	KeyCtrlShfEnd           string
+	KeyAltShfHome           string
+	KeyAltShfEnd            string
+	KeyMetaShfHome          string
+	KeyMetaShfEnd           string
+	EnablePaste             string // bracketed paste mode
+	DisablePaste            string
+	PasteStart              string
+	PasteEnd                string
+	Modifiers               int
+	InsertChar              string // string to insert a character (ich1)
+	AutoMargin              bool   // true if writing to last cell in line advances
+	TrueColor               bool   // true if the terminal supports direct color
+	CursorDefault           string
+	CursorBlinkingBlock     string
+	CursorSteadyBlock       string
+	CursorBlinkingUnderline string
+	CursorSteadyUnderline   string
+	CursorBlinkingBar       string
+	CursorSteadyBar         string
+	EnterUrl                string
+	ExitUrl                 string
+	SetWindowSize           string
+	EnableFocusReporting    string
+	DisableFocusReporting   string
+	DisableAutoMargin       string // smam
+	EnableAutoMargin        string // rmam
 }
 
 const (
@@ -227,112 +241,61 @@ const (
 	ModifiersXTerm = 1
 )
 
-type stackElem struct {
-	s     string
-	i     int
-	isStr bool
-	isInt bool
-}
+type stack []interface{}
 
-type stack []stackElem
-
-func (st stack) Push(v string) stack {
-	e := stackElem{
-		s:     v,
-		isStr: true,
-	}
-	return append(st, e)
-}
-
-func (st stack) Pop() (string, stack) {
-	v := ""
-	if len(st) > 0 {
-		e := st[len(st)-1]
-		st = st[:len(st)-1]
-		if e.isStr {
-			v = e.s
+func (st stack) Push(v interface{}) stack {
+	if b, ok := v.(bool); ok {
+		if b {
+			return append(st, 1)
 		} else {
-			v = strconv.Itoa(e.i)
+			return append(st, 0)
 		}
 	}
-	return v, st
+	return append(st, v)
 }
 
+func (st stack) PopString() (string, stack) {
+	if len(st) > 0 {
+		e := st[len(st)-1]
+		var s string
+		switch v := e.(type) {
+		case int:
+			s = strconv.Itoa(v)
+		case string:
+			s = v
+		}
+		return s, st[:len(st)-1]
+	}
+	return "", st
+
+}
 func (st stack) PopInt() (int, stack) {
 	if len(st) > 0 {
 		e := st[len(st)-1]
-		st = st[:len(st)-1]
-		if e.isInt {
-			return e.i, st
-		} else if e.isStr {
-			// If the string that was pushed was the representation
-			// of a number e.g. '123', then return the number. If the
-			// conversion doesn't work, assume the string pushed was
-			// intended to return, as an int, the ascii representation
-			// of the (one and only) character.
-			i, err := strconv.Atoi(e.s)
-			if err == nil {
-				return i, st
-			} else if len(e.s) >= 1 {
-				return int(e.s[0]), st
-			}
+		var i int
+		switch v := e.(type) {
+		case int:
+			i = v
+		case string:
+			i, _ = strconv.Atoi(v)
 		}
+		return i, st[:len(st)-1]
 	}
 	return 0, st
 }
 
-func (st stack) PopBool() (bool, stack) {
-	if len(st) > 0 {
-		e := st[len(st)-1]
-		st = st[:len(st)-1]
-		if e.isStr {
-			if e.s == "1" {
-				return true, st
-			}
-			return false, st
-		} else if e.i == 1 {
-			return true, st
-		} else {
-			return false, st
-		}
-	}
-	return false, st
-}
-
-func (st stack) PushInt(i int) stack {
-	e := stackElem{
-		i:     i,
-		isInt: true,
-	}
-	return append(st, e)
-}
-
-func (st stack) PushBool(i bool) stack {
-	if i {
-		return st.PushInt(1)
-	}
-	return st.PushInt(0)
-}
-
 // static vars
 var svars [26]string
 
-// paramsBuffer handles some persistent state for TParam.  Technically we
-// could probably dispense with this, but caching buffer arrays gives us
-// a nice little performance boost.  Furthermore, we know that TParam is
-// rarely (never?) called re-entrantly, so we can just reuse the same
-// buffers, making it thread-safe by stashing a lock.
 type paramsBuffer struct {
 	out bytes.Buffer
 	buf bytes.Buffer
-	lk  sync.Mutex
 }
 
 // Start initializes the params buffer with the initial string data.
 // It also locks the paramsBuffer.  The caller must call End() when
 // finished.
 func (pb *paramsBuffer) Start(s string) {
-	pb.lk.Lock()
 	pb.out.Reset()
 	pb.buf.Reset()
 	pb.buf.WriteString(s)
@@ -341,7 +304,6 @@ func (pb *paramsBuffer) Start(s string) {
 // End returns the final output from TParam, but it also releases the lock.
 func (pb *paramsBuffer) End() string {
 	s := pb.out.String()
-	pb.lk.Unlock()
 	return s
 }
 
@@ -360,18 +322,16 @@ func (pb *paramsBuffer) PutString(s string) {
 	pb.out.WriteString(s)
 }
 
-var pb = &paramsBuffer{}
-
 // TParm takes a terminfo parameterized string, such as setaf or cup, and
 // evaluates the string, and returns the result with the parameter
 // applied.
-func (t *Terminfo) TParm(s string, p ...int) string {
+func (t *Terminfo) TParm(s string, p ...interface{}) string {
 	var stk stack
-	var a, b string
+	var a string
 	var ai, bi int
-	var ab bool
 	var dvars [26]string
-	var params [9]int
+	var params [9]interface{}
+	var pb = &paramsBuffer{}
 
 	pb.Start(s)
 
@@ -381,7 +341,13 @@ func (t *Terminfo) TParm(s string, p ...int) string {
 		params[i] = p[i]
 	}
 
-	nest := 0
+	const (
+		emit = iota
+		toEnd
+		toElse
+	)
+
+	skip := emit
 
 	for {
 
@@ -391,7 +357,9 @@ func (t *Terminfo) TParm(s string, p ...int) string {
 		}
 
 		if ch != '%' {
-			pb.PutCh(ch)
+			if skip == emit {
+				pb.PutCh(ch)
+			}
 			continue
 		}
 
@@ -400,31 +368,51 @@ func (t *Terminfo) TParm(s string, p ...int) string {
 			// XXX Error
 			break
 		}
+		if skip == toEnd {
+			if ch == ';' {
+				skip = emit
+			}
+			continue
+		} else if skip == toElse {
+			if ch == 'e' || ch == ';' {
+				skip = emit
+			}
+			continue
+		}
 
 		switch ch {
 		case '%': // quoted %
 			pb.PutCh(ch)
 
 		case 'i': // increment both parameters (ANSI cup support)
-			params[0]++
-			params[1]++
+			if i, ok := params[0].(int); ok {
+				params[0] = i + 1
+			}
+			if i, ok := params[1].(int); ok {
+				params[1] = i + 1
+			}
 
-		case 'c', 's':
-			// NB: these, and 'd' below are special cased for
+		case 's':
+			// NB: 's', 'c', and 'd' below are special cased for
 			// efficiency.  They could be handled by the richer
 			// format support below, less efficiently.
-			a, stk = stk.Pop()
+			a, stk = stk.PopString()
 			pb.PutString(a)
 
+		case 'c':
+			// Integer as special character.
+			ai, stk = stk.PopInt()
+			pb.PutCh(byte(ai))
+
 		case 'd':
 			ai, stk = stk.PopInt()
 			pb.PutString(strconv.Itoa(ai))
 
-		case '0', '1', '2', '3', '4', 'x', 'X', 'o', ':':
+		case '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'x', 'X', 'o', ':':
 			// This is pretty suboptimal, but this is rarely used.
 			// None of the mainstream terminals use any of this,
 			// and it would surprise me if this code is ever
-			// executed outside of test cases.
+			// executed outside test cases.
 			f := "%"
 			if ch == ':' {
 				ch, _ = pb.NextCh()
@@ -442,26 +430,29 @@ func (t *Terminfo) TParm(s string, p ...int) string {
 			case 'd', 'x', 'X', 'o':
 				ai, stk = stk.PopInt()
 				pb.PutString(fmt.Sprintf(f, ai))
-			case 'c', 's':
-				a, stk = stk.Pop()
+			case 's':
+				a, stk = stk.PopString()
 				pb.PutString(fmt.Sprintf(f, a))
+			case 'c':
+				ai, stk = stk.PopInt()
+				pb.PutString(fmt.Sprintf(f, ai))
 			}
 
 		case 'p': // push parameter
 			ch, _ = pb.NextCh()
 			ai = int(ch - '1')
 			if ai >= 0 && ai < len(params) {
-				stk = stk.PushInt(params[ai])
+				stk = stk.Push(params[ai])
 			} else {
-				stk = stk.PushInt(0)
+				stk = stk.Push(0)
 			}
 
 		case 'P': // pop & store variable
 			ch, _ = pb.NextCh()
 			if ch >= 'A' && ch <= 'Z' {
-				svars[int(ch-'A')], stk = stk.Pop()
+				svars[int(ch-'A')], stk = stk.PopString()
 			} else if ch >= 'a' && ch <= 'z' {
-				dvars[int(ch-'a')], stk = stk.Pop()
+				dvars[int(ch-'a')], stk = stk.PopString()
 			}
 
 		case 'g': // recall & push variable
@@ -472,10 +463,10 @@ func (t *Terminfo) TParm(s string, p ...int) string {
 				stk = stk.Push(dvars[int(ch-'a')])
 			}
 
-		case '\'': // push(char)
+		case '\'': // push(char) - the integer value of it
 			ch, _ = pb.NextCh()
-			pb.NextCh() // must be ' but we don't check
-			stk = stk.Push(string(ch))
+			_, _ = pb.NextCh() // must be ' but we don't check
+			stk = stk.Push(int(ch))
 
 		case '{': // push(int)
 			ai = 0
@@ -486,147 +477,99 @@ func (t *Terminfo) TParm(s string, p ...int) string {
 				ch, _ = pb.NextCh()
 			}
 			// ch must be '}' but no verification
-			stk = stk.PushInt(ai)
+			stk = stk.Push(ai)
 
 		case 'l': // push(strlen(pop))
-			a, stk = stk.Pop()
-			stk = stk.PushInt(len(a))
+			a, stk = stk.PopString()
+			stk = stk.Push(len(a))
 
 		case '+':
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai + bi)
+			stk = stk.Push(ai + bi)
 
 		case '-':
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai - bi)
+			stk = stk.Push(ai - bi)
 
 		case '*':
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai * bi)
+			stk = stk.Push(ai * bi)
 
 		case '/':
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
 			if bi != 0 {
-				stk = stk.PushInt(ai / bi)
+				stk = stk.Push(ai / bi)
 			} else {
-				stk = stk.PushInt(0)
+				stk = stk.Push(0)
 			}
 
 		case 'm': // push(pop mod pop)
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
 			if bi != 0 {
-				stk = stk.PushInt(ai % bi)
+				stk = stk.Push(ai % bi)
 			} else {
-				stk = stk.PushInt(0)
+				stk = stk.Push(0)
 			}
 
 		case '&': // AND
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai & bi)
+			stk = stk.Push(ai & bi)
 
 		case '|': // OR
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai | bi)
+			stk = stk.Push(ai | bi)
 
 		case '^': // XOR
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai ^ bi)
+			stk = stk.Push(ai ^ bi)
 
 		case '~': // bit complement
 			ai, stk = stk.PopInt()
-			stk = stk.PushInt(ai ^ -1)
+			stk = stk.Push(ai ^ -1)
 
 		case '!': // logical NOT
 			ai, stk = stk.PopInt()
-			stk = stk.PushBool(ai != 0)
+			stk = stk.Push(ai == 0)
 
-		case '=': // numeric compare or string compare
-			b, stk = stk.Pop()
-			a, stk = stk.Pop()
-			stk = stk.PushBool(a == b)
+		case '=': // numeric compare
+			bi, stk = stk.PopInt()
+			ai, stk = stk.PopInt()
+			stk = stk.Push(ai == bi)
 
 		case '>': // greater than, numeric
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushBool(ai > bi)
+			stk = stk.Push(ai > bi)
 
 		case '<': // less than, numeric
 			bi, stk = stk.PopInt()
 			ai, stk = stk.PopInt()
-			stk = stk.PushBool(ai < bi)
+			stk = stk.Push(ai < bi)
 
 		case '?': // start conditional
 
+		case ';':
+			skip = emit
+
 		case 't':
-			ab, stk = stk.PopBool()
-			if ab {
-				// just keep going
-				break
-			}
-			nest = 0
-		ifloop:
-			// this loop consumes everything until we hit our else,
-			// or the end of the conditional
-			for {
-				ch, err = pb.NextCh()
-				if err != nil {
-					break
-				}
-				if ch != '%' {
-					continue
-				}
-				ch, _ = pb.NextCh()
-				switch ch {
-				case ';':
-					if nest == 0 {
-						break ifloop
-					}
-					nest--
-				case '?':
-					nest++
-				case 'e':
-					if nest == 0 {
-						break ifloop
-					}
-				}
+			ai, stk = stk.PopInt()
+			if ai == 0 {
+				skip = toElse
 			}
 
 		case 'e':
-			// if we got here, it means we didn't use the else
-			// in the 't' case above, and we should skip until
-			// the end of the conditional
-			nest = 0
-		elloop:
-			for {
-				ch, err = pb.NextCh()
-				if err != nil {
-					break
-				}
-				if ch != '%' {
-					continue
-				}
-				ch, _ = pb.NextCh()
-				switch ch {
-				case ';':
-					if nest == 0 {
-						break elloop
-					}
-					nest--
-				case '?':
-					nest++
-				}
-			}
-
-		case ';': // endif
+			skip = toEnd
 
+		default:
+			pb.PutString("%" + string(ch))
 		}
 	}
 
@@ -643,15 +586,15 @@ func (t *Terminfo) TPuts(w io.Writer, s string) {
 		beg := strings.Index(s, "$<")
 		if beg < 0 {
 			// Most strings don't need padding, which is good news!
-			io.WriteString(w, s)
+			_, _ = io.WriteString(w, s)
 			return
 		}
-		io.WriteString(w, s[:beg])
+		_, _ = io.WriteString(w, s[:beg])
 		s = s[beg+2:]
 		end := strings.Index(s, ">")
 		if end < 0 {
 			// unterminated.. just emit bytes unadulterated
-			io.WriteString(w, "$<"+s)
+			_, _ = io.WriteString(w, "$<"+s)
 			return
 		}
 		val := s[:end]
@@ -722,7 +665,6 @@ func (t *Terminfo) TColor(fi, bi int) string {
 var (
 	dblock    sync.Mutex
 	terminfos = make(map[string]*Terminfo)
-	aliases   = make(map[string]string)
 )
 
 // AddTerminfo can be called to register a new Terminfo entry.
